## Code for "Important airborne lidar metrics of canopy structure for estimating snow interception"
## Micah Russell, Jan U.H. Eitel, Timothy E. Link, and Carlos A. Silva
## Last updated 10/11/21 by Micah Russell, mrussell@western.edu
## Publication pending

###############################################################################
###Generate Terrestrial Lidar Scanning (TLS) volumes for select trees##########

##Code adapted from Eitel, J.U.H. (2019), jeitel@uidaho.edu

###############################################################################

require(rgl)            ## load rgl package which allows plotting 3d data
require(alphashape3d)   ## load alphashapte3d package package which allows plotting 3d data
require(magic)          
require(rgl)
require(raster)
require(sp)
require(plot3D)

##NOTE:PROCESS EACH TREE ONE AT A TIME

#Designate Area of Interest (AOI) files for snow off and snow on samples

tls_snow_on_dec_aoi =  choose.files("INSERT FILE PATH") 
tls_snow_on_jan_aoi = choose.files("INSERT FILE PATH") 
tls_snow_off_aoi = choose.files("INSERT FILE PATH") 

################### Process TLS Snow On: December #######################

tls_snow_on_dec = read.table(tls_snow_on_dec_aoi, sep = " ", header = FALSE) 
colnames(tls_snow_on_dec) = c("x", "y", "z") #ignores intensity
tls_snow_on_dec <- subset(tls_snow_on_dec, select = -c(4:7))

##SUBSET UPPER 75% of TREE##  
subset_tls_snow_on_dec = which(tls_snow_on_dec$z > min(tls_snow_on_dec$z)+(0.25*(max(tls_snow_on_dec$z)- min(tls_snow_on_dec$z)))) 
subset_tls_snow_on_dec = tls_snow_on_dec[subset_tls_snow_on_dec, ]

subset_tls_snow_on_dec = as.matrix(subset_tls_snow_on_dec) 

ashape_3d_dec_50mm = ashape3d(subset_tls_snow_on_dec, alpha = 0.05)

ashape.volume.dec.50mm = volume_ashape3d(ashape_3d_dec_50mm)

ashape.volume.dec.50mm

################### Process TLS Snow On: January #######################

tls_snow_on_jan = read.table(tls_snow_on_jan_aoi, sep = " ", header = FALSE) 
colnames(tls_snow_on_jan) = c("x", "y", "z") #ignores intensity
tls_snow_on_jan <- subset(tls_snow_on_jan, select = -c(4:7))

##SUBSET UPPER 75% of TREE##  
subset_tls_snow_on_jan = which(tls_snow_on_jan$z > min(tls_snow_on_jan$z)+(0.25*(max(tls_snow_on_jan$z)- min(tls_snow_on_jan$z)))) 
subset_tls_snow_on_jan = tls_snow_on_jan[subset_tls_snow_on_jan, ]

subset_tls_snow_on_jan = as.matrix(subset_tls_snow_on_jan) 

ashape_3d_jan_50mm = ashape3d(subset_tls_snow_on_jan, alpha = 0.05)

ashape.volume.jan.50mm = volume_ashape3d(ashape_3d_jan_50mm)

ashape.volume.jan.50mm

################### Process TLS Snow Off: October #######################

tls_snow_off_aoi = read.table(tls_snow_on_jan_aoi, sep = " ", header = FALSE) 
colnames(tls_snow_off_aoi) = c("x", "y", "z") #ignores intensity
tls_snow_off_aoi <- subset(tls_snow_off_aoi, select = -c(4:7))

##SUBSET UPPER 75% of TREE##  
subset_tls_snow_off_aoi = which(tls_snow_off_aoi$z > min(tls_snow_off_aoi$z)+(0.25*(max(tls_snow_off_aoi$z)- min(tls_snow_off_aoi$z)))) 
subset_tls_snow_off_aoi = tls_snow_off_aoi[subset_tls_snow_off_aoi, ]

subset_tls_snow_off_aoi = as.matrix(subset_tls_snow_off_aoi) 

ashape_3d_off_50mm = ashape3d(subset_tls_snow_off_aoi, alpha = 0.05)

ashape.volume.off.50mm = volume_ashape3d(ashape_3d_off_50mm)

ashape.volume.off.50mm

##########################################################################
##########Generate Canopy Metrics from Aerial Laser Scanning (ALS)########

##Uses Areas of Interest (AOIs) derived manually from ALS scene

##Code adapted from Silva, C.A. (2019), c.silva@ufl.edu

##########################################################################

cha<-function(x,y){
  chull(x,y)->i
  return(areapl(cbind(x[i],y[i])))
}

HStats<-function(xyzid) {
  #print(xyzid[1,4])
  require(alphashape3d)
  require(plyr)
  require(splancs)
  library(plyr)
  require(rgdal)
  require(rLiDAR)
  require(maptools)
  require(sp)
  library(mixtools)
  
  #plot(xyzid[,1:2], main=unique(xyzid[,5]))
  "skewness" <- function(x, na.rm = FALSE) {
    if (is.matrix(x)) 
      apply(x, 2, skewness, na.rm = na.rm)
    else if (is.vector(x)) {
      if (na.rm) 
        x <- x[!is.na(x)]
      n <- length(x)
      (sum((x - mean(x))^3)/n)/(sum((x - mean(x))^2)/n)^(3/2)
    }
    else if (is.data.frame(x)) 
      sapply(x, skewness, na.rm = na.rm)
    else skewness(as.vector(x), na.rm = na.rm)
  }
  "kurtosis" <- function(x, na.rm = FALSE) {
    if (is.matrix(x)) 
      apply(x, 2, kurtosis, na.rm = na.rm)
    else if (is.vector(x)) {
      if (na.rm) 
        x <- x[!is.na(x)]
      n <- length(x)
      n * sum((x - mean(x))^4)/(sum((x - mean(x))^2)^2)
    }
    else if (is.data.frame(x)) 
      sapply(x, kurtosis, na.rm = na.rm)
    else kurtosis(as.vector(x), na.rm = na.rm)
  }
  CBH<-tryCatch({
    model<- normalmixEM(x=xyzid[,3], k=2)
    index.higher <- which.max(model$mu)
    CHB<-model$mu[index.higher]-model$sigma[index.higher]*1.5
  },error = function(err) { return(CBH=quantile(xyzid[,3],0.25))})
  n<-nrow(xyzid)
  xcenter<-mean(xyzid[,1])
  xmin<-min(xyzid[,1])
  xmax<-max(xyzid[,1])
  ycenter<-mean(xyzid[,2])
  ymin<-min(xyzid[,2])
  ymax<-max(xyzid[,2])
  hmax<-max(xyzid[,3])
  hmin<-min(xyzid[,3])
  hmean<-mean(xyzid[,3])
  hskew<-skewness(xyzid[,3],na.rm = T)
  hkur<-kurtosis(xyzid[,3],na.rm = T)
  
  if (CBH < 2) {CBH<-hmax/2
  #CBH<-quantile(xyzid[,3],0.25)
  }
  
  if ( nrow(xyzid)==1) { hsd<-0; hvar<-0} else {
    hsd<-sd(as.numeric(xyzid[,3]), na.rm=T)
    hvar<-var(as.numeric(xyzid[,3]), na.rm=T) }
  
  CA<-cha(xyzid[,1],xyzid[,2])
  CL<-hmax-CBH
  CRatio<-CBH/hmax
  CRad<-sqrt(CA/pi)
  hq<-quantile(xyzid[,3], c(seq(0.05,1,0.05)[-20],0.99))
  CDens<-(nrow(subset(xyzid,xyzid[,3] >= CBH))/ nrow(xyzid))*100
  
  imax<-max(xyzid[,4])
  imin<-min(xyzid[,4])
  imean<-mean(xyzid[,4])
  if ( nrow(xyzid)==1) { isd<-0; ivar<-0} else {
    isd<-sd(as.numeric(xyzid[,4]), na.rm=T)
    ivar<-var(as.numeric(xyzid[,4]), na.rm=T) }
  iq<-quantile(xyzid[,4], c(seq(0.05,1,0.05)[-20],0.99))
  
  #if (n <= 3) { CV=c(0,0)} else {
  # CV<-tryCatch({paste0(chullLiDAR3D(xyzid[,c(1:3,5)],plotit = TRUE)[,2:3])}, 
  #             error = function(err) { return(CV=c(0,0))})}
  
  #CV above CBH
  if (n <= 3) { CV=c(0,0)} else {
    xyzid0<-xyzid[,c(1:3,5)]
    CV<-tryCatch({paste0(chullLiDAR3D(subset(xyzid0,xyzid0[,3]<CBH),plotit = TRUE)[,2:3])},
                 error = function(err) { return(CV=c(0,0))})}
  
  xyzid2<-xyzid[,1:3]
  xyzid2[,1]<-min(xyzid[,1])- xyzid[,1]
  xyzid2[,2]<-min(xyzid[,2])- xyzid[,2]
  
  if (n <= 3) { AlphaShape1=0; AlphaShape0.5=0; AlphaShape0.25=0} else {
    AlphaShape1<-tryCatch({paste0(volume_ashape3d(ashape3d(as.matrix(xyzid2),alpha=0.75, pert=TRUE)))}, 
                          error = function(err) { return(AlphaShape1=0)})
    
    AlphaShape0.5<-tryCatch({paste0(volume_ashape3d(ashape3d(as.matrix(xyzid2),alpha=0.5, pert=TRUE)))}, 
                            error = function(err) { return(AlphaShape0.5=0)})
    
    AlphaShape0.25<-tryCatch({paste0(volume_ashape3d(ashape3d(as.matrix(xyzid2),alpha=0.25, pert=TRUE)))}, 
                             error = function(err) { return(AlphaShape0.25=0)})}
  
  stats<-c(n,xcenter,xmin,xmax,ycenter,ymin,ymax,hmax,hmin,hmean,hsd,hvar,hkur,hskew,hq,imax,imin,imean,isd,ivar,iq,CDens,CL,CBH,CRatio,CRad,CA,CV,ASV0.25=AlphaShape0.25,
           ASV0.5=AlphaShape0.5,ASV1=AlphaShape1)
  return(stats)
}

runMetrics<-function(xyzid) {
  Stats<-ddply(data.frame(xyzid), "treeID",HStats)
  colnames(Stats)<-c("TreeID","Nreturns","E","Emin","Emax","N","Nmin","Nmax","HMAX","HMIN","HMEAN",
                     "HSD","HVAR","HKUR","HSKEW","H5TH","H10TH","H15TH","H20TH","H25TH","H30TH","H35TH","H40TH",
                     "H45TH","H50TH","H55TH","H60TH","H65TH","H70TH","H75TH","H80TH","H85TH","H90TH","H95TH","H99TH",
                     "IMAX","IMIN","IMEAN","ISD","IVAR","IH5TH","I10TH","I15TH","I20TH","I25TH","I30TH","I35TH","I40TH",
                     "I45TH","I50TH","I55TH","I60TH","I65TH","I70TH","I75TH","I80TH","I85TH","I90TH","I95TH","I99TH",
                     "CDens","CL","CBH","CRatio","CRad","CPA","CSA","CV","ASV0.25","ASV0.5","ASV1")
  return(Stats)
}

#head(trees@data)
#shp<-ITD_shp
CrownMetricsLiDAR<-function(trees){
  metrics<-runMetrics(trees@data[,c("X","Y","Z","Intensity","treeID")])
  indx <- sapply(metrics, is.list)
  metrics[indx] <- lapply(metrics[indx], function(x) as.numeric(as.character(x)))
  indx <- sapply(metrics, is.character)
  metrics[indx] <- lapply(metrics[indx], function(x) as.numeric(as.character(x)))
  return(metrics)
}

##try one tree at a time

als_tree1 = choose.files("INSERT FILE PATH") 
als_tree1 = read.table(als_tree1, sep = " ", header = FALSE) 
colnames(als_tree1) = c("x", "y", "z", "i") #ignores intensity
als_tree1 <- subset(als_tree1, select = c(1:4))
als_tree1$treeID = 1
head(als_tree1)
als_tree1 = as.matrix(als_tree1)
metrics_Tree1<-runMetrics(als_tree1)
head(metrics_Tree1)

##compile canopy metrics for each tree into a table

new.crown.metrics.table = rbind(metrics_Tree1, metrics_Tree2...)
new.crown.metrics.table = as.matrix(new.crown.metrics.table)
head(new.crown.metrics.table) 

new.crown.metrics.table.export = write.csv(new.crown.metrics.table, "INSERT FILE PATH")

#new.crown.metrics.table = data.frame(new.crown.metrics.table)

##VISUALIZATION CODE FOR ONE TREE AT A TIME

## CODE TO FORCE XYZ TO ZERO SCALE; WILL NEED TO DO THIS TO new.crown.metrics.table TO PLOT ABLINES

#subset_als_tree1$x0 = subset_als_tree1$x - min(subset_als_tree1$x) 
#subset_als_tree1$y0 = subset_als_tree1$y - min(subset_als_tree1$y) 
#subset_als_tree1$z0 = subset_als_tree1$z - min(subset_als_tree1$z) 
#head(subset_als_tree1)

##TREE 1

par(mfrow=c(1,3))

hist(als_tree1[,3], freq=FALSE,main="hist of heights (z)",xlab="height (z)")
lines(density(als_tree1[,3]), col="red", lwd=2)

plot(als_tree1[,1], als_tree1[,3], xlab="x", ylab="z", pch=16, col="forestgreen", main="x view")
abline(h=new.crown.metrics.table[1,20], col="blue", lwd=2) # p25th
abline(h=new.crown.metrics.table[1,25], col="green", lwd=2) # p50th
abline(h=new.crown.metrics.table[1,30], col="orange", lwd=2) # p75th
abline(h=new.crown.metrics.table[1,35], col="red", lwd=2) #p99th
abline(h=new.crown.metrics.table[1,9], col = "purple", lwd = 2) #HMAX
abline(h=new.crown.metrics.table[1,63], col = "brown", lwd = 2) #CBH

##########################################################################
############Calculate Novel Metrics From ALS##############################

##Code adapted from Eitel, J.U.H. (2019), jeitel@uidaho.edu

##########################################################################

####################################
### User input starts here
####################################

### Lidar laser footprint. Note - value you chose here affects your final results (e.g., the larger the footprint, the more likely it is that some of the returns overlap)
lidar_laser_footprint = 0.15  

### provide file path for lidar txt file for one tree canopy
ufc = read.table("INSERT FILE PATH", sep = "", header = FALSE)    

###################################
### End input 
###################################

dimension = dim(ufc)[1]

unobstructed_returns_nd = dimension

unobstructed_returns = dimension

for(ii in 1:dimension){
  
  minix = ufc[ii,1]-(lidar_laser_footprint/2)         ### Calculate the surface area of a given return with where x and y are assumed to be the center of the return
  maxix = ufc[ii,1]+(lidar_laser_footprint/2)
  
  miniy = ufc[ii,2]-(lidar_laser_footprint/2)
  maxiy = ufc[ii,2]+(lidar_laser_footprint/2)
  
  extract = which(ufc[,1]>= minix & ufc[,1]<= maxix)  ### How many other returns are within the laser footprint (i.e., what is the overlap?)
  
  subset = ufc[extract,]
  
  extract = which(subset[,2]>= miniy & subset[,2]<= maxiy)
  
  subset = subset[extract,]
  
  subtract_nd = (dim(subset)[1]-1)/dim(subset)[1]     ### uses no decay function
  
  if(dim(subset)[1] == 1){
    
    subtract = 0 
    
  }else{
    
    subtract =0.0498*exp(0.5*dim(subset)[1])         ### decay function (decay function is a little bit arbitrary but somewhat accounts for the fact that the liklyhood of intercepting snow exponentially decreases with the number of canopy layers)
    
  }
  
  unobstructed_returns_nd = unobstructed_returns_nd - subtract_nd   ### No decay (nd) function - if there is overlap, account for it by subtracting it from the overall number of returns
  
  unobstructed_returns = unobstructed_returns - subtract            ### Considers decay function
  
} 

all_returns = dimension

all_returns                 ### All returns

unobstructed_returns        ### Returns with decay function

unobstructed_returns_nd     ### Returns with no decay (nd) function 

##########################################################################
############RUN RANDOM FOREST MODELS######################################

##Code adapted from Meddens, A. (2019), arjan.meddens@wsu.edu

#####################################################################
#- 1. Open data
#####################################################################

##Read in CSV file that contains TLS volumes, Standard Canopy Metrics, and Novel Metrics

Mydata <- read.csv("INSERT FILE PATH", header = T, sep = ",")

#####################################################################
# Load packages
#####################################################################
require(rgdal)
require(spatial)
require(raster)
require(sp)
library(rgl)
require(yaImpute)
require(randomForest)
require(rfUtilities)

#####################################################################
# Read data #
#####################################################################
y.arr = Mydata[,2]
x1.arr  = Mydata[,3]
x2.arr  = Mydata[,4]
x3.arr  = Mydata[,5]
x4.arr  = Mydata[,6]
x5.arr  = Mydata[,7]
x6.arr  = Mydata[,8]
x7.arr  = Mydata[,9]
x8.arr  = Mydata[,10]
x9.arr  = Mydata[,11]
x10.arr  = Mydata[,12]
x11.arr  = Mydata[,13]
x12.arr  = Mydata[,14]
x13.arr  = Mydata[,15]
x14.arr  = Mydata[,16]
x15.arr  = Mydata[,17]
x16.arr  = Mydata[,18]
x17.arr  = Mydata[,19]
x18.arr  = Mydata[,20]
x19.arr  = Mydata[,21]
x20.arr  = Mydata[,22]
x21.arr  = Mydata[,23]
x22.arr  = Mydata[,24]
x23.arr  = Mydata[,25]
x24.arr  = Mydata[,26]
x25.arr  = Mydata[,27]
x26.arr  = Mydata[,28]
x27.arr  = Mydata[,29]
x28.arr  = Mydata[,30]
x29.arr  = Mydata[,31]

dat.x = cbind(x1.arr,x2.arr,x3.arr, x4.arr, x5.arr, x6.arr, x7.arr, x8.arr, x9.arr, x10.arr, x11.arr, x12.arr, x13.arr, x14.arr, x15.arr, x16.arr, x17.arr, x18.arr, x19.arr, x20.arr, x21.arr, x22.arr, x23.arr, x24.arr, x25.arr, x26.arr, x27.arr, x28.arr, x29.arr)
colnames(dat.x) <- c('UNOB', 'Percent_UNOB', 'UNOBDK', 'Percent_UNOBDK', 'HMAX','HMIN','HRANGE', 'HQR', 'HMEAN', 'HSD', 'HKUR', 'HSKEW', 'H25TH', 'H50TH', 'H75TH', 'H90TH', 'H95TH', 'H99TH', 'CDENS',  'CL', 'CBH', 'CRATIO', 'CRAD', 'CPA', 'CSA', 'CV', 'ASV0.25', 'ASV0.5', 'ASV0.75')
dat.x = data.frame(dat.x)
attach(dat.x)

## Run Randomforest 
rf.1 <- randomForest(dat.x, y = y.arr, importance=TRUE, na.action=na.omit,proximity=TRUE,type=regression,ntree=1000)
print(rf.1)

## Potential RF tools to utilize
names(rf.1)

## Examine Variable Importance
rf.1$importanceSD
barplot(rf.1$importance[,2])

## Plot predicted versus observed values
rf.1$call
rf.1$predicted
y.arr
plot(y.arr,rf.1$predicted, ylab = "predicted volume (m3)", xlab = "measured volume (m3)", cex.lab = 1.5, lwd=3,  xlim=c(0, 40), ylim=c(0, 40))
abline(0,1,lwd=6,lty=2,col='dark grey')

## Calculate R2 and RMSE 
model.stats <- function(obs, pred)
{
  R2 = cor(pred, obs)^2
  MBE = mean(obs) - mean(pred)
  pMBE = MBE/mean(obs)
  RMSE = sqrt(mean((obs - pred)^2))
  pRMSE = RMSE/mean(obs)
  MAE = mean(abs(obs - pred))  	# Mean Absolute error, see Gorelik thesis
  ME = mean(obs - pred)    	# Mean Error, see Gorelik thesis
  return = list(R2=R2,MBE=MBE,pMBE=pMBE,RMSE=RMSE,pRMSE=pRMSE,MAE=MAE,ME=ME)
}
stats = model.stats(y.arr,rf.1$predicted)
stats

#-- Checking cross correlations --##
plot((dat.x))
cor.table = cor(dat.x)
round(cor.table, 2)
cor.table = as.matrix(cor.table)

##removing HMAX, HMEAN, HMODE, ETOP, EMIN, EMAX, EWIDTH and half of percentiles due to frequently high correlation with other variables

dat.x.2 = dat.x[-c(3,5,7,9,11,13,14,21,23,25,27,29,31,33,35,38,39, 46, 47, 49)] 
rf.2 <- randomForest(dat.x.2, y = y.arr, importance=TRUE, na.action=na.omit,proximity=TRUE,type=regression,ntree=1000)
print(rf.2)
rf.2$call
rf.2$predicted
y.arr
plot(y.arr,rf.2$predicted, ylab = "predicted volume (m3)", xlab = "measured volume (m3)", cex.lab = 1.5, lwd=3,  xlim=c(0, 40), ylim=c(0, 40))
abline(0,1,lwd=6,lty=2,col='dark grey')

model.stats <- function(obs, pred)
{
  R2 = cor(pred, obs)^2
  MBE = mean(obs) - mean(pred)
  pMBE = MBE/mean(obs)
  RMSE = sqrt(mean((obs - pred)^2))
  pRMSE = RMSE/mean(obs)
  MAE = mean(abs(obs - pred))  	# Mean Absolute error, see Gorelik thesis
  ME = mean(obs - pred)    	# Mean Error, see Gorelik thesis
  return = list(R2=R2,MBE=MBE,pMBE=pMBE,RMSE=RMSE,pRMSE=pRMSE,MAE=MAE,ME=ME)
}
stats = model.stats(y.arr,rf.2$predicted)
stats

#if (flag_varsel == 1) {
#create an empty matrix
Names <- names(dat.x.2)
iterations<-1000
VarSel<-matrix(ncol=length(Names), nrow=iterations)
#bootstrapp the rf.modelSel
for (i in 1:iterations){
  seed_n<-sample(1:28, 1, replace=FALSE)  
  fit_msel<-rf.modelSel(dat.x.2, y.arr, imp.scale = "mir",
                        final.model = FALSE, seed = seed_n, ntree=28, parsimony=0.03, plot.imp=TRUE)
  varimp<-fit_msel$imp
  vardat<-varimp[,1]
  
  df<-t(vardat)
  VarSel[i,] <- c(df)
  
  names(vardat) <- colnames(dat.x.2)
  Sys.sleep(0.1)
  print(sort(vardat,decreasing = T))
  print(i)
  #print(vardat)
  #print(VarSel)
  
}
# sum up across the 1000  runs  
#include<-VarSel>0.2
include=VaSel
summaryinclude<-colSums(include)
summary.rf.selboot<-cbind(Names,summaryinclude)
summary.rf.selboot = data.frame(summary.rf.selboot)
var.csv = write.csv(summary.rf.selboot, "C:/Users/russe/Desktop/SNOW II/Variable_Selection_Table_10cm_COVs_Removed.csv")

#-----------------------------------------------
#colnames(dat.img) changed above

######################
# Rules for selection: 
# 1. r<0.95, choose one: Here Red and Blue are super correlated, so Red goes.
# 2. rf.modelSel with Varsel include > 0.2
######################

###############################################################################
###Extract Leaf Area Index (LAI) from each study tree##########################

##Code adapted from Eitel, J.U.H. (2021); jeitel@uidaho.edu

###############################################################################

require(lidR)
require(leafR)

### Step 1: Create canopy height model (DSM) based on lidar data. Canopy height model can then be used to create shapefiles (e.g., using QGIS) 
###         associated with the trees of interest.

### Read in lidar data which are usually saved as .las or .laz files
las = readLAS("INSERT FILE PATH")         

### Set morphological filter details - see details in paper by Zhang et al (2003). 

ws = seq(3,12, 3)                                  ##   Sequence of windows sizes to be used in filtering ground returns                                      
th = seq(0.4, 1.5, length.out = length(ws))        ##   Sequence of threshold heights above the parameterized ground surface 

las = classify_ground(las, pmf(ws, th))            ##   Classify ground vs vegetation returns

plot(las, color = "Classification")                ##   Visual assessment of classification accuracy

dtm = grid_terrain(las, algorithm = tin(), res = 1)       ## Interpolates the ground points and creates a rasterized digital terrain model.  

dsm = grid_canopy(las, algorithm = p2r(), res = 1)                     ## Creates a digital surface model (DSM) 
## Different approaches are available such as p2r and pitfree   

CHM = dsm - dtm                                                       ## Create a gridded canopy height model by subtracting the DSM from the DTM 

plot(CHM)

writeRaster(CHM,'C:/Field season 2021/Experiments/Nokes UAV/10_05_2020_34/10_05_2020/chm/chm.tif')        ### Export chm to hard drive 

### Step 2: Create an LAI raster based on lidar (.las. or .laz) input files

VOXELS_LAD = lad.voxels("INSERT FILE PATH", grain.size = 10, k=1)   ### Provide file path associated with the lidar file you want to use to calculate LAI

### For more details on the lad.voxel function (e.g., meaning of grain.size and k), see ?lad.voxel

?lad.voxels

lai_raster = lai.raster(VOXELS_LAD)       ### Create LAI raster   
x11()                                     ### Display LAI raster
plot(lai_raster)

### Step 3: Extract mean LAI values for a given tree based on LAI raster created in step 2 and shapefile created in step 1

## Read in shapefile you created in step 1 (e.g., using QGIS) 
tree1 = readOGR("INSERT FILE PATH")         

plot(lai_raster, 1)
plot(tree1, add = TRUE)          ### Plot shapefile over canopy height model

tree1_lai = extract(lai_raster, tree1)             ### extract LAI values associated with a given tree and calculate the mean
tree1_lai = data.frame(tree1_lai)
tree1_lai_mean = mean(tree1_lai[,1], na.rm = TRUE)

tree1_lai_mean

###############################################################################
###Predict TLS Volume From LAI For Comparison to Random Forest Model Results###

##Code adapted from Eitel, J.U.H. (2021); jeitel@uidaho.edu

###############################################################################

require(sjstats)
require(lme4) 
require(MuMIn) 
require(arm)
require(randomForest) 
require(broom) 
require(nlme)           

##Read in csv file with site #, TLS volumes, LAI values, etc.
ufc=read.csv("INSERT FILE PATH")
model1 = lmer(volume ~ LAI+(1|time_id)+(1|ufc[,1]), REML = FALSE, data = ufc, na.action=na.omit)  
tt = p_value(model1) 
tt
r.squaredGLMM(model1)

plot(fitted(model1), ylab = ~ "predicted volume" (m^3), xlab = ~ "observed volume" (m^3), cex.lab = 1.5, lwd=3,  xlim=c(0,30), ylim=c(0, 4))
abline(0,1,lwd=6,lty=2,col='dark grey')